// --- --- --- --- --- --- --- ---
// --- --- --- Precaching
// --- --- --- --- --- --- --- ---

PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "spell_pumpkin_mirv_bits_red"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "spell_pumpkin_mirv_bits_blue"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "rd_robot_explosion"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "unusual_festive_lights_glow1"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "unusual_festive_lights_glow3"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "healthgained_red"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "healthgained_blu"}) // Blu lmao
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "arm_detonate_sparks"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "teleportedin_red"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "teleportedin_blue"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "electrocuted_gibbed_red"})
PrecacheEntityFromTable({classname = "info_particle_system", effect_name = "electrocuted_gibbed_blue"})
PrecacheScriptSound("items/gunpickup2.wav")
PrecacheScriptSound("weapons/airstrike_small_explosion_03.wav")



// --- --- --- --- --- --- --- ---
// --- --- --- Spawn entities & Setup
// --- --- --- --- --- --- --- ---

spawnGroup <- SpawnEntityFromTable("tf_robot_destruction_spawn_group", {
	targetname = "robot_spawn_group"
	group_number = 0
	"OnRobotsRespawn": "tf_robot_destruction_robot_spawn,RunScriptCode,NotifyRobotSpawned(),0,-1"
})
EntityOutputs.AddOutput(spawnGroup, "OnRobotsRespawn", "!self", "RunScriptCode", "NetProps.SetPropInt(self,`m_nState`,4)", 0, -1)
// We are abusing the fact that the robot spawns' spawn group can be set
// from the script when it is first executed. This way there's less to
// do within hammer.


teamControlPointMaster <- Entities.FindByClassname(null, "team_control_point_master")
if (!teamControlPointMaster) {
	teamControlPointMaster <- SpawnEntityFromTable("team_control_point_master", {
		targetname = "team_control_point_master"
		caplayout = "0 1 2 3 4 5 6" // ToDo
		cpm_restrict_team_cap_win = 1
		switch_teams = 1
		custom_position_y = 0.825
	})
}

// There can be only one tf_gamerules anyway and one is spawned on map spawn
NetProps.SetPropInt(Entities.FindByClassname(null, "tf_gamerules"), "m_nHudType", 2)

::limbo <- self



// --- --- --- --- --- --- --- ---
// --- --- --- Core spawner
// --- --- --- --- --- --- --- ---

/**
 * !global
 *
 * name
 * - SpawnBonusPack
 *
 * description
 * - Spawns a bonus pack
 *
 * params
 * - props: {
 * - - teamNum: 2|3
 * - - origin: Vector
 * - - basevelocity?: Vector
 * - - value?: number
 * - }
 */
::SpawnCore <- function(props) {
	if (!("basevelocity" in props)) props.basevelocity <- Vector(RandomFloat(-100,100),RandomFloat(-100,100),250) // NoT A cIrcLe
	if (!("value" in props)) props.value <- 1

	// Create entities
	local core = SpawnEntityFromTable("item_powerup_rune", {
		powerup_model = "models/bots/bot_worker/bot_worker_powercore.mdl",
		origin = props.origin.ToKVString(),
		basevelocity = props.basevelocity.ToKVString(),
		skin = props.teamNum - 2,
		TeamNum = 4,
		rendermode = 5,
		"OnCacheInteraction": "!self,RunScriptCode,handleTouch(),0,-1"
		"OnUser1": "!self,RunScriptCode,triggerFadePhase(),5,1"
		"OnUser2": "!self,RunScriptCode,triggerFadePhase(),0,-1"
	})
	local particle = SpawnEntityFromTable("info_particle_system", {
		effect_name = [
			"unusual_festive_lights_glow3" // Blue particle
			"unusual_festive_lights_glow1" // Red particle
		][props.teamNum-2],
		start_active = 1
	})

	// Setup core
	EntFireByHandle(core, "FireUser1", "", 0, null, null)
	NetProps.SetPropEntity(particle, "moveparent", core)
	core.ValidateScriptScope()
	local coreScope = core.GetScriptScope()
	coreScope.teamNum <- props.teamNum
	coreScope.value <- props.value
	coreScope.touched <- false
	coreScope.particle <- particle
	coreScope.onPickupParticleEffect <-[
		"spell_pumpkin_mirv_bits_red",
		"spell_pumpkin_mirv_bits_blue"
	][props.teamNum-2]
	coreScope.addPointsFunction <- [
		::AddToBlueScore,
		::AddToRedScore
	][props.teamNum-2]

	// Setup fading animation
	coreScope.fadePhase <- 0
	coreScope.fadePhaseAmt <- 5
	coreScope.fadePhaseDuration <- 1
	coreScope.triggerFadePhase <- function() {
		if (fadePhase == fadePhaseAmt) {
			particle.Kill()
			self.Kill()
		}
		else {
			EntFireByHandle(self, "AddOutput", "renderamt "+(255-(fadePhase*255/fadePhaseAmt)), 0, null, null)
			EntFireByHandle(self, "FireUser2", "", fadePhaseDuration, null, null)
			fadePhase++
		}
	}

	// Handle core touch
	coreScope.handleTouch <- function() {
		if (activator.GetTeam() != teamNum && !touched) {
			touched = true
			DispatchParticleEffect(onPickupParticleEffect, self.GetOrigin()+Vector(0,0,16), Vector(0,0,0))
			EmitSoundEx({
				sound_name = "items/gunpickup2.wav",
				origin = self.GetOrigin()+Vector(0,0,16),
				sound_level = 70
			})
			addPointsFunction(value)
			particle.Kill()
			self.Kill()
		}
	}

}



// --- --- --- --- --- --- --- ---
// --- --- --- Score handling
// --- --- --- --- --- --- --- ---

redScore <- 0
blueScore <- 0
roundOver <- false
maxScore <- NetProps.GetPropInt(self, "m_nMaxPoints")

/**
 * !global
 *
 * name
 * - AddToRedScore
 *
 * params
 * - amt: number
 */
::AddToRedScore <- (function(amt) {
	if (redScore < maxScore && !roundOver) {
		redScore += amt
		if (redScore > maxScore) redScore = maxScore
		if (redScore == maxScore) {
			roundOver = true
			local finaleDuration = NetProps.GetPropFloat(self, "m_flFinaleLength")
			NetProps.SetPropFloat(self, "m_flRedFinaleEndTime", Time()+finaleDuration)
			EntFireByHandle(teamControlPointMaster, "SetWinner", "2", finaleDuration, null, null)
			printl("Red wins!")
		}
		NetProps.SetPropInt(self, "m_nRedScore", redScore)
		NetProps.SetPropInt(self, "m_nRedTargetPoints", redScore)
	}
}).bindenv(this)

/**
 * !global
 *
 * name
 * - AddToBlueScore
 *
 * params
 * - amt: number
 */
::AddToBlueScore <- (function(amt) {
	if (blueScore < maxScore && !roundOver) {
		blueScore += amt
		if (blueScore > maxScore) blueScore = maxScore
		if (blueScore == maxScore) {
			roundOver = true
			local finaleDuration = NetProps.GetPropFloat(self, "m_flFinaleLength")
			NetProps.SetPropFloat(self, "m_flBlueFinaleEndTime", Time()+finaleDuration)
			EntFireByHandle(teamControlPointMaster, "SetWinner", "3", finaleDuration, null, null)
		}
		NetProps.SetPropInt(self, "m_nBlueScore", blueScore)
		NetProps.SetPropInt(self, "m_nBlueTargetPoints", blueScore)
	}
}).bindenv(this)



// --- --- --- --- --- --- --- ---
// --- --- --- Control points handling
// --- --- --- --- --- --- --- ---

redControlPoints <- 0
blueControlPoints <- 0

redTimer <- SpawnEntityFromTable("logic_timer", {
	StartDisabled = 1,
	"OnTimer": "!self,RunScriptCode,::AddToRedScore(1),0,-1"
})
blueTimer <- SpawnEntityFromTable("logic_timer", {
	StartDisabled = 1,
	"OnTimer": "!self,RunScriptCode,::AddToBlueScore(1),0,-1"
})

/**
 * !global
 *
 * name
 * - AddControlPointsToRed
 *
 * params
 * - amt: number
 */
::AddControlPointsToRed <- function(amt) {
	redControlPoints += amt
	if (redControlPoints > 0) {
		EntFireByHandle(redTimer, "RefireTime", ""+(1.0/redControlPoints), 0, null, null)
		EntFireByHandle(redTimer, "Enable", "", 0, null, null)
	}
	else {
		EntFireByHandle(redTimer, "Disable", "", 0, null, null)
		EntFireByHandle(redTimer, "ResetTimer", "", 0, null, null)
	}
}.bindenv(this)

/**
 * !global
 *
 * name
 * - AddControlPointsToBlue
 *
 * params
 * - amt: number
 */
::AddControlPointsToBlue <- function(amt) {
	blueControlPoints += amt
	if (blueControlPoints > 0) {
		EntFireByHandle(blueTimer, "RefireTime", ""+(1.0/blueControlPoints), 0, null, null)
		EntFireByHandle(blueTimer, "Enable", "", 0, null, null)
	}
	else {
		EntFireByHandle(blueTimer, "Disable", "", 0, null, null)
		EntFireByHandle(blueTimer, "ResetTimer", "", 0, null, null)
	}
}.bindenv(this)



// --- --- --- --- --- --- --- ---
// --- --- --- OnTakeDamage
// --- --- --- --- --- --- --- ---

ClearGameEventCallbacks()
function OnScriptHook_OnTakeDamage(info) {
	info.const_entity.ValidateScriptScope()
	local scope = info.const_entity.GetScriptScope()
	if ("handleDamage" in scope) scope.handleDamage(info)
}
__CollectGameEventCallbacks(this)