id <- RandomInt(10000,99999)

// Setup teamNum by reading the Group Name
teamNum <- {
	red = 2
	blue = 3
}[NetProps.GetPropString(self, "m_spawnData.m_pszGroupName")]
// Override the Group Name with the one created by rd_logic.nut
NetProps.SetPropString(self, "m_spawnData.m_pszGroupName", "robot_spawn_group")

// Setup every other property
maxHp <- NetProps.GetPropInt(self, "m_spawnData.m_nRobotHealth") || 200
NetProps.SetPropInt(self, "m_spawnData.m_nRobotHealth", 99999)
hp <- maxHp
coresAmt <- NetProps.GetPropInt(self, "m_spawnData.m_nNumGibs") || 5

// Declare sone names
name <- self.GetName()
robotName <- name+"_robot"
destinationName <- name+"_target"

// Spawn the path_track that the robot follows (the "destination")
robotDestination <- SpawnEntityFromTable("path_track", {
	targetname = destinationName
	"OnPass": "!self,RunScriptCode,handleOnPass(),0,-1"
})
// Prevent a crash by having the destination path_track loop on itself
NetProps.SetPropEntity(robotDestination, "m_pnext", robotDestination)
// Get the destination's scope and set it up
robotDestination.ValidateScriptScope()
robotDestinationScope <- robotDestination.GetScriptScope()
robotDestinationScope.robotSpawnScope <- this
robotDestinationScope.target <- null
robotDestinationScope.handleOnPass <- function() {
	if (target) {
		target.ValidateScriptScope()
		local targetScope = target.GetScriptScope()
		if ("handleArrival" in targetScope) targetScope.handleArrival(robotSpawnScope)
	}
}
// Update the Start Path to the destination, but before that memorize the original one so that the robot can be teleported there on spawn
spawnPoint <- Entities.FindByName(null, NetProps.GetPropString(self, "m_spawnData.m_pszPathName"))
NetProps.SetPropString(self, "m_spawnData.m_pszPathName", destinationName)

// Declare some slots
robot <- null
hitbox <- null
latestDamageInfo <- null
deaths <- 0

// Regen data
stagger <- 0
isRecovering <- false
maxStagger <- 100

/**
 * name
 * - NotifyRobotSpawned
 */
function NotifyRobotSpawned() {

	// Find robot
	robot = Entities.FindByName(null, robotName)

	// Setup Hitbox
	hitbox = SpawnEntityFromTable("obj_sentrygun", {})
	EntFireByHandle(hitbox, "SetParent", robotName, 0, null, null)
	EntFireByHandle(hitbox, "SetParentAttachment", "wheel", 0, null, null)
	EntFireByHandle(hitbox, "SetHealth", ""+id, 0, null, null)
	EntFireByHandle(hitbox, "Hide", null, 0, null, null)
	hitbox.SetSolidFlags(2)
	hitbox.ValidateScriptScope()
	local hitboxScope = hitbox.GetScriptScope()
	hitboxScope.handleDamage <- handleDamage.bindenv(this)
	hitboxScope.handleRecovery <- handleRecovery.bindenv(this)

	// Update robot
	robot.SetSolidFlags(4) // No collisions nor hitbox
	SetTeam(teamNum)
	if (spawnPoint) {
		LocateTo(spawnPoint, true)
	}

	// FireUser1 hook (on robot spawn)
	EntFireByHandle(self, "FireUser1", "", 0, null, null)
}

/**
 * name
 * - GoTo
 *
 * params
 * - entity: path_track
 */
function GoTo(entity) {
	robotDestination.SetAbsOrigin(entity.GetOrigin())
	robotDestinationScope.target = entity
}

/**
 * name
 * - TeleportTo
 *
 * params
 * - entity: path_track
 */
function TeleportTo(entity, silent = false) {
	entity.ValidateScriptScope()
	local entityScope = entity.GetScriptScope()
	if ("handleTeleport" in entityScope) entityScope.handleTeleport(this, silent)
	else robot.SetAbsOrigin(entity.GetOrigin())
}

/**
 * name
 * - LocateTo
 *
 * description
 * - Combines GoTo and TeleportTo
 *
 * params
 * - entity: path_track
 */
function LocateTo(entity, silent = false) {
	GoTo(entity)
	TeleportTo(entity, silent)
}

/**
 * !private
 *
 * name
 * - handleDamage
 *
 * param
 * - info: damageInfo // As seen in OnTakeDamage
 */
function handleDamage(info) {
	if (info.attacker.GetClassname() != "player") printl("Robot was damaged by not a player")
	if (info.damage == 0 || info.attacker.GetClassname() != "player") {
		info.early_out = true
		return
	}
	latestDamageInfo <- info
	if (info.attacker.GetTeam() == this.teamNum) {
		info.early_out = true
	}
	else {
		// Recovery handling
		if (!isRecovering) {
			isRecovering = true
			EntFireByHandle(self, "RunScriptCode", "handleRecovery()", 1, null, null)
			stagger += 10
		}
		stagger += info.damage
		if (stagger > maxStagger) stagger = maxStagger
		robot.TakeDamage(0, 1, info.attacker)
		SetHp(hp - info.damage)
		// Keep the hitbox's health to its id
		EntFireByHandle(hitbox, "SetHealth", ""+id, 0, null, null)
	}
}

function handleRecovery() {
	playSparkles()
	stagger -= 10
	if (stagger < 0) stagger = 0
	if (stagger > 0) EntFireByHandle(self, "RunScriptCode", "handleRecovery()", 1, null, null)
	else {
		if (hp < maxHp) {
			SetHp(hp+10)
			DispatchParticleEffect(teamNum == 2 ? "healthgained_red" : "healthgained_blu", robot.GetOrigin()+Vector(RandomFloat(-1,1)*12, RandomFloat(-1,1)*12, RandomFloat(1,1.5)*48), Vector(0,0,0))
		}
		if (hp > maxHp) hp = maxHp
		if (hp == maxHp) isRecovering = false
		else EntFireByHandle(self, "RunScriptCode", "handleRecovery()", 1, null, null)
	}
}

function playSparkles() {
	local amtPlayed = 0
	local attemptsToPlay = RandomInt(1,(5*(1-(hp/maxHp))).tointeger()+1)
	for (local i = attemptsToPlay; i > 0; i--) {
		local randomValue = RandomFloat(0,1)
		local cmpValue = hp/maxHp
		if (randomValue > cmpValue) {
			amtPlayed++
			DispatchParticleEffect("arm_detonate_sparks", robot.GetOrigin()+Vector(RandomFloat(-1,1)*12, RandomFloat(-1,1)*12, RandomFloat(0.5,1.25)*48), Vector(0,0,0))
		}
	}
}

/**
 * name
 * - EmitSound
 *
 * description
 * - Make sure the soundName is cached;
 * - Use PrecacheScriptSound(soundName) if necessary.
 *
 * param
 * - soundName: string
 */
function EmitSound(soundName) {
	EmitSoundEx({
		sound_name = soundName,
		origin = robot.GetOrigin(),
		sound_level = 70
	})
}

/**
 * name
 * - SetHp
 *
 * description
 * - Sets the robot's hp;
 * - If amt <= 0 then the robot dies;
 * - Cannot go past maxHp.
 *
 * params
 * - amt: number
 * - panic: boolean
 */
function SetHp(amt) {
	hp = amt
	if (hp > maxHp) hp = maxHp
	if (hp <= 0) TriggerDeath()
	else {
		local color = ((205*hp/maxHp)+50).tointeger()
		EntFireByHandle(robot, "color", ""+color+" "+color+" "+color, 0, null, null)
	}
}

/**
 * name
 * - SetTeam
 *
 * params
 * - num: 2|3
 */
function SetTeam(num) {
	teamNum = num
	robot.SetSkin(teamNum-2)
	robot.SetTeam(teamNum)
	hitbox.SetTeam(teamNum)
}

/**
 * name
 * - TriggerDeath
 *
 * description
 * - Triggers the robot's death and team switch
 */
function TriggerDeath() {
	deaths++
	id <- RandomInt(10000,99999)
	EntFireByHandle(hitbox, "SetHealth", ""+id, 0, null, null)

	// Death cosmetics
	EmitSound("weapons/airstrike_small_explosion_03.wav")
	DispatchParticleEffect("rd_robot_explosion", robot.GetOrigin(), Vector(90,0,0))
	local prop = SpawnEntityFromTable("prop_physics_override", {
		model = "models/bots/bot_worker/bot_worker2.mdl",
		massScale = 0.02,
		origin = robot.GetOrigin().ToKVString(),
		angles = (robot.GetAbsAngles()+Vector(0,0,0)).ToKVString(),
		skin = teamNum - 2,
		rendercolor = "75 75 75",
		"OnUser1#1": "!self,AddOutput,renderfx 5,6,1",
		"OnUser1#2": "!self,Kill,,10,1"
	})
	prop.SetSolidFlags(2)
	EntFireByHandle(prop, "FireUser1", null, 0, null, null)

	// Cores spawning
	for (local i = 0; i < coresAmt; i++) {
		::SpawnCore({
			teamNum = teamNum,
			origin = robot.GetOrigin()+Vector(0,0,32),
			value = 5,
		})
	}

	// Respawn
	SetHp(maxHp)
	stagger = 0
	LocateTo(::limbo, true)

	// FireUser4 hook (on robot death)
	EntFireByHandle(self, "FireUser4", "", 0, null, null)
}
